/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.Comment;
import org.wso2.registry.LogEntry;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.DatabaseConstants;

import java.sql.*;
import java.sql.Date;
import java.util.*;

public class CommentsDAO {

    public void addComment(long artifactID, String userID, Comment comment, Connection conn)
            throws SQLException {

        String sql = "INSERT INTO COMMENTS (AID, USER_ID, COMMENT_TEXT, COMMENTED_TIME) " +
                        "VALUES (?, ?, ?, ?)";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.setString(2, userID);
        s.setString(3, comment.getCommentText());
        s.setDate(4, new Date(now));

        s.executeUpdate();
    }

    public Comment[] getComments(String path, Connection conn) throws SQLException {

        String sql = "SELECT C.COMMENT_TEXT, C.USER_ID, C.COMMENTED_TIME FROM COMMENTS C, ARTIFACTS A WHERE A.PATH=? AND A.AID=C.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);

        List commentList = new ArrayList();
        ResultSet results = s.executeQuery();
        while(results.next()) {
            Comment comment = new Comment();
            comment.setCommentText(results.getString(DatabaseConstants.COMMENT_TEXT_FIELD));
            comment.setCommentedUser(results.getString(DatabaseConstants.COMMENTED_USER_ID_FIELD));
            comment.setCommentedTime(results.getDate(DatabaseConstants.COMMENTED_TIME_FIELD));

            commentList.add(comment);
        }

        Comment[] comments = (Comment[]) commentList.toArray(new Comment[commentList.size()]);
        return comments;
    }

    public void removeComments(long artifactID, Connection conn) throws SQLException {

        String sql = "DELETE FROM COMMENTS WHERE AID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.executeUpdate();
    }

    public Comment getComment(long commentID, Connection conn) throws SQLException {

        String sql = "SELECT * FROM COMMENTS WHERE CM_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, commentID);

        ResultSet result = s.executeQuery();

        if (result.next()) {
            Comment comment = new Comment();
            comment.setCommentText(result.getString(DatabaseConstants.COMMENT_TEXT_FIELD));
            comment.setCommentedUser(result.getString(DatabaseConstants.COMMENTED_USER_ID_FIELD));
            comment.setCommentedTime(result.getDate(DatabaseConstants.COMMENTED_TIME_FIELD));

            return comment;
        }

        return null;        
    }

    public long[] getCommentIDs(String resourcePath, Connection conn) throws SQLException {

        String sql = "SELECT C.CM_ID FROM COMMENTS C, ARTIFACTS A WHERE A.PATH=? AND A.AID=C.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, resourcePath);

        List commentIDList = new ArrayList();
        ResultSet results = s.executeQuery();
        while(results.next()) {
            long commentID = results.getLong(DatabaseConstants.COMMENT_ID_FIELD);
            commentIDList.add(new Long(commentID));
        }

        long[] commentIDs = new long[commentIDList.size()];
        for (int i = 0; i < commentIDs.length; i++) {
            commentIDs[i] = ((Long) commentIDList.get(i)).longValue();
        }
        return commentIDs;
    }

    public List getLogs(String resourcePath, String userName,
                        java.util.Date from, java.util.Date to, Connection conn)
            throws SQLException {

        ArtifactDAO artifactDAO = new ArtifactDAO();

        long aid = -1;
        aid = artifactDAO.getArtifactID(resourcePath, conn);

        String sql = "SELECT A.PATH, C.USER_ID, C.COMMENTED_TIME FROM ARTIFATCS A, COMMENTS C";

        boolean criteriaStarted = false;

        if (aid != -1) {
            if (criteriaStarted) {
                sql += " AND AID=?";
            } else {
                criteriaStarted = true;
                sql += " WHERE AID=?";
            }
        }

        if (userName != null) {
            if (criteriaStarted) {
                sql += " AND USER_ID=?";
            } else {
                criteriaStarted = true;
                sql += " WHERE USER_ID=?";
            }
        }

        if (from != null) {
            if (criteriaStarted) {
                sql += " AND COMMENTED_TIME<?";
            } else {
                criteriaStarted = true;
                sql += " WHERE USER_ID<?";
            }
        }

        if (from != null) {
            if (criteriaStarted) {
                sql += " AND COMMENTED_TIME>?";
            } else {
                criteriaStarted = true;
                sql += " WHERE USER_ID>?";
            }
        }

        if (to != null) {
            if (criteriaStarted) {
                sql += " AND COMMENTED_TIME<?";
            } else {
                criteriaStarted = true;
                sql += " WHERE USER_ID<?";
            }
        }

        PreparedStatement s = conn.prepareStatement(sql);

        int paramNumber = 1;

        if (aid != -1) {
            s.setLong(paramNumber, aid);
            paramNumber++;
        }

        if (userName != null) {
            s.setString(paramNumber, userName);
            paramNumber++;
        }

        if (from != null) {
            s.setTimestamp(paramNumber, new Timestamp(from.getTime()));
            paramNumber++;
        }

        if (to != null) {
            s.setTimestamp(paramNumber, new Timestamp(to.getTime()));
            paramNumber++;
        }

        ResultSet results = s.executeQuery();

        List resultList = new ArrayList();
        while (results.next()) {
            LogEntry logEntry = new LogEntry();
            logEntry.setResourcePath(results.getString(DatabaseConstants.PATH_FIELD));
            logEntry.setUserName(results.getString(DatabaseConstants.USER_ID_FIELD));
            logEntry.setDate(new Date(results.getTimestamp(DatabaseConstants.COMMENTED_TIME_FIELD).getTime()));
            logEntry.setAction(LogEntry.COMMENT);
            logEntry.setActionData(results.getString(DatabaseConstants.COMMENT_TEXT_FIELD));

            resultList.add(logEntry);
        }

        return resultList;
    }

    public String getResourcePathOfComment(long commentID, Connection conn) throws SQLException {

        String sql = "SELECT A.PATH FROM ARTIFACTS A, COMMENTS C WHERE A.AID=C.AID AND C.CM_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, commentID);

        ResultSet result = s.executeQuery();
        if (result.next()) {
            String resourcePath = result.getString(DatabaseConstants.PATH_FIELD);
            return resourcePath;
        }

        return null;
    }
}
